﻿using System;
using System.Collections.Generic;
using System.Text;
using Bonea.PropertyRegister.MunicipalityProperty.Entities;
using Bonea.PropertyRegister.Common;
using Bonea.PropertyRegister.DataAccess;
using NHibernate;

namespace Bonea.PropertyRegister.MunicipalityProperty.EntityRules
{
    public class PropertyCertificateRules
    {
        private PropertyCertificateRules()
        {
        }

        public static void ValidateSave(PropertyCertificate certificate)
        {
            List<RuleError> errors = new List<RuleError>();

            if (certificate.Number <= 0)
            {
                errors.Add(new RuleError("Number", "Невалиден номер на акт"));
            }
            if (String.IsNullOrEmpty(certificate.RegisterNumber))
            {
                errors.Add(new RuleError("RegisterNumber", "Невалиден регистър на акт"));
            }
            if (String.IsNullOrEmpty(certificate.FileNumber))
            {
                errors.Add(new RuleError("FileNumber", "Невалидно досие на акт"));
            }
            if (String.IsNullOrEmpty(certificate.LawReason))
            {
                errors.Add(new RuleError("LawReason",
                    "Невалидно правно основание на акт"));
            }

            long certificateCount = GetPropertyCertificateCount(
                certificate.Number);
            if ((certificate.IsNew && (certificateCount > 0)) ||
                ((certificate.IsNew == false) && (certificateCount > 1)))
            {
                errors.Add(new RuleError("Number", "Вече съществува акт със същия номер"));
            }

            if (errors.Count > 0)
            {
                throw new RuleException(errors);
            }
        }

        private static long GetPropertyCertificateCount(int number)
        {
            SessionManager dbSession = SessionManagerFactory.GetInstance();
            IQuery searchQuery = dbSession.Session.CreateQuery(
                "select count(pc) from PropertyCertificate as pc " +
                "where pc.Number = :Number");
            searchQuery.SetInt32("Number", number);

            long count = (long)searchQuery.UniqueResult();

            return count;
        }

        public static void ValidateDelete(PropertyCertificate certificate)
        {
            if (!((certificate.Status.StatusCode == PropertyCertificateStatusCodes.Created) ||
            (certificate.Status.StatusCode == PropertyCertificateStatusCodes.ApprovalRejected)))
            {
                throw new RuleException("NoProperty",
                     "Акта се намира в състояние в което неможе да бъде изтрит");
            }
        }

        public static void ValidateChangeStatus(PropertyCertificate certificate,
            PropertyCertificateStatusCodes newStatus)
        {
            switch (newStatus)
            {
                case PropertyCertificateStatusCodes.Created:
                    throw new RuleException("Status",
                         "Акта неможе да бъде установяван в състояние 'Създаден'");
                case PropertyCertificateStatusCodes.SendedForApproval:
                    if (!((certificate.Status.StatusCode == PropertyCertificateStatusCodes.Created) ||
                    (certificate.Status.StatusCode == PropertyCertificateStatusCodes.ApprovalRejected) ||
                    (certificate.Status.StatusCode == PropertyCertificateStatusCodes.SignRejected)))
                    {
                        throw new RuleException("Status",
                             "Акта неможе да бъде установяван в състояние 'За утвърждаване' ако той не е 'Нов акт' 'Отхвърлен второ ниво' или 'Подписването отказано'");
                    }
                    break;
                case PropertyCertificateStatusCodes.Approved:
                    if (certificate.Status.StatusCode != PropertyCertificateStatusCodes.SendedForApproval)
                    {
                        throw new RuleException("Status",
                             "Акта неможе да бъде установяван в състояние 'Утвърден второ ниво' ако той не е 'За утвърждаване'");
                    }
                    break;
                case PropertyCertificateStatusCodes.ApprovalRejected:
                    if (certificate.Status.StatusCode != PropertyCertificateStatusCodes.SendedForApproval)
                    {
                        throw new RuleException("Status",
                             "Акта неможе да бъде установяван в състояние 'Отхвърлен второ ниво' ако той не е 'За утвърждаване'");
                    }
                    break;
                case PropertyCertificateStatusCodes.Signed:
                    if (certificate.Status.StatusCode != PropertyCertificateStatusCodes.Approved)
                    {
                        throw new RuleException("Status",
                             "Акта неможе да бъде установяван в състояние 'Подписан' ако той не е 'Утвърден второ ниво'");
                    }
                    break;
                case PropertyCertificateStatusCodes.SignRejected:
                    if (certificate.Status.StatusCode != PropertyCertificateStatusCodes.Approved)
                    {
                        throw new RuleException("Status",
                             "Акта неможе да бъде установяван в състояние 'Подписването отказано' ако той не е 'Утвърден второ ниво'");
                    }
                    break;
                case PropertyCertificateStatusCodes.Registered:
                    if (certificate.Status.StatusCode != PropertyCertificateStatusCodes.Signed)
                    {
                        throw new RuleException("Status",
                             "Акта неможе да бъде установяван в състояние 'Вписан' ако той не е 'Подписан'");
                    }
                    break;
                case PropertyCertificateStatusCodes.RegistrationRejected:
                    if (certificate.Status.StatusCode != PropertyCertificateStatusCodes.Signed)
                    {
                        throw new RuleException("Status",
                             "Акта неможе да бъде установяван в състояние 'Отхвърлена заявка за вписване' ако той не е 'Подписан'");
                    }
                    break;
                default: throw new ArgumentException("Неизвестен статус на акт");
            }
        }
    }
}
