﻿using System;
using System.Data;
using System.Web;
using System.Collections;
using System.Collections.Generic;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.ComponentModel;
using Bonea.PropertyRegister.MunicipalityProperty.EntityRules;
using Bonea.PropertyRegister.DataAccess;
using Bonea.PropertyRegister.MunicipalityProperty.Entities;
using System.Security.Permissions;
using Microsoft.Web.Services3;
using NHibernate;
using Bonea.PropertyRegister.Common;
using Bonea.PropertyRegister.WebServiceCommon;
using System.Threading;
using System.Text;

namespace Bonea.PropertyRegister.MunicipalityProperty.WebService
{
    /// <summary>
    /// Summary description for PropertyCertificate
    /// </summary>
    [WebService(Namespace =
        "http://bonea.com/PropertyRegister/MunicipalityProperty/PropertyCertificate")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    public class PropertyCertificateService : ServiceBase
    {
        private const String INVALID_WORKSPACE = "Нямате права за работа с работното " +
            "пространство, към което принадлежи акта";
        
        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateCreate, Authenticated = true)]
        public PropertyCertificate New(int[] propertyIDList)
        {
            PropertyCertificate certificate = new PropertyCertificate();

            certificate.DateCreated = DateTime.Now;
            certificate.Status = SessionContext.Session.Load<PropertyCertificateStatus>(
                AppConst.DefaultPropertyCertificateStatusID);
            certificate.DocumentType = SessionContext.Session.Load<PropertyCertificateType>(
                AppConst.DefaultPropertyCertificateTypeID);
            certificate.Issuer = GetDefaultIssuer();

            AppUser emptyUser = SessionContext.Session.Load<AppUser>(AppConst.EmptyUserID);
            certificate.CreatedByUser = CurrentUser;
            certificate.ApprovedByUser = emptyUser;
            certificate.SignedByUser = emptyUser;
            certificate.LastUpdatedByUser = emptyUser;
            certificate.WorkspaceID = CurrentUser.Workspace.ID;

            Property property;
            foreach (int propertyID in propertyIDList)
            {
                property = SessionContext.Session.Load<Property>(propertyID);
                certificate.PropertyList.Add(property);

                certificate.Description += property.PropType.PropTypeDescr
                    + Environment.NewLine;
                certificate.Borders += property.PropBorders + Environment.NewLine;
                certificate.Location += BuildPropertyLocation(property)
                    + Environment.NewLine;
            }

            return certificate;
        }

        private PropertyCertificateIssuer GetDefaultIssuer()
        {
            IList<PropertyCertificateIssuer> issuerList = GetIssuerList();
            if (issuerList.Count > 0)
            {
                return issuerList[0];
            }
            return null;
        }

        private string BuildPropertyLocation(Property property)
        {
            StringBuilder location = new StringBuilder();
            AddPropertyLocationItem(location, property.PropLand, "местност:");
            AddPropertyLocationItem(location, property.PropPopulArea, "маселено място:");
            AddPropertyLocationItem(location, property.PropMunicip, "община:");
            AddPropertyLocationItem(location, property.PropRegion, "област:");
            AddPropertyLocationItem(location, property.PropDistr, "квартал:");
            AddPropertyLocationItem(location, property.PropBul, "булевард:");
            AddPropertyLocationItem(location, property.PropBulNum, "№");
            AddPropertyLocationItem(location, property.PropStr, "улица:");
            AddPropertyLocationItem(location, property.PropStrNum, "№");

            return location.ToString();
        }

        private const char WORD_SEPARATOR = ' ';

        private void AddPropertyLocationItem(StringBuilder location, string item,
            string itemHeading)
        {
            if ((String.IsNullOrEmpty(item) == false) && (item.Trim() != String.Empty))
            {
                location.Append(itemHeading);
                location.Append(WORD_SEPARATOR);
                location.Append(item);
                location.Append(WORD_SEPARATOR);
            }
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateView, Authenticated = true)]
        public PropertyCertificate Load(int ID)
        {
            PropertyCertificate entity = SessionContext.Session.Load<PropertyCertificate>(ID);
            if (entity.WorkspaceID != CurrentUser.Workspace.ID)
            {
                throw new InvalidWorkspaceException(
                    INVALID_WORKSPACE);
            }

            return entity;
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateSave, Authenticated = true)]
        public void Save(PropertyCertificate certificate)
        {
            PropertyCertificateRules.ValidateSave(certificate);

            if (certificate.WorkspaceID != CurrentUser.Workspace.ID)
            {
                throw new InvalidWorkspaceException(INVALID_WORKSPACE);
            }

            certificate.LastUpdatedByUser = CurrentUser;

            SessionContext.BeginTransaction(IsolationLevel.ReadCommitted);
            try
            {
                SessionContext.Session.SaveOrUpdate(certificate);

                SessionContext.CommitTransaction();
                SessionContext.Session.Flush();
            }
            catch
            {
                SessionContext.RollbackTransaction();
                SessionContext.Session.Clear();
                throw;
            }
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateDelete, Authenticated = true)]
        public void DeleteByID(int ID)
        {
            PropertyCertificate entity = SessionContext.Session.Load<PropertyCertificate>(ID);

            Delete(entity);
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateDelete, Authenticated = true)]
        public void Delete(PropertyCertificate certificate)
        {
            PropertyCertificateRules.ValidateDelete(certificate);

            if (certificate.WorkspaceID != CurrentUser.Workspace.ID)
            {
                throw new InvalidWorkspaceException(INVALID_WORKSPACE);
            }

            LogPropertyCertificate logCertificate = BuildLogPropertyCertificate(
                certificate);

            SessionContext.BeginTransaction(IsolationLevel.ReadCommitted);
            try
            {
                //SessionContext.Session.Save(logCertificate);
                SessionContext.Session.Delete(certificate);

                SessionContext.CommitTransaction();
                SessionContext.Session.Flush();
            }
            catch
            {
                SessionContext.RollbackTransaction();
                SessionContext.Session.Clear();
                throw;
            }
        }

        private LogPropertyCertificate BuildLogPropertyCertificate(
            PropertyCertificate certificate)
        {
            LogPropertyCertificate logCertificate = new LogPropertyCertificate();
            logCertificate.LogDate = DateTime.Now;
            logCertificate.LogActionType = "D";
            logCertificate.UserActualize = UserManager.GetAppUser(
                Thread.CurrentPrincipal.Identity.Name);
            logCertificate.ID = certificate.ID;
            logCertificate.Status = certificate.Status;
            logCertificate.DocumentType = certificate.DocumentType;
            logCertificate.DateCreated = certificate.DateCreated;
            logCertificate.RegisterNumber = certificate.RegisterNumber;
            logCertificate.CardIndexNumber = certificate.CardIndexNumber;
            logCertificate.FileNumber = certificate.FileNumber;
            logCertificate.LawReason = certificate.LawReason;
            logCertificate.Description = certificate.Description;
            logCertificate.Location = certificate.Location;
            logCertificate.Borders = certificate.Borders;
            logCertificate.TaxAssesment = certificate.TaxAssesment;
            logCertificate.MarketAssesment = certificate.MarketAssesment;
            logCertificate.ExOwners = certificate.ExOwners;
            logCertificate.Owners = certificate.Owners;
            logCertificate.ExCertificates = certificate.ExCertificates;
            logCertificate.Orders = certificate.Orders;
            logCertificate.Notes = certificate.Notes;
            logCertificate.RejectRemarks = certificate.RejectRemarks;
            logCertificate.RegistrationNumber = certificate.RegistrationNumber;
            logCertificate.RegistrationDate = certificate.RegistrationDate;
            logCertificate.CreatedByUser = certificate.CreatedByUser;
            logCertificate.ApprovedByUser = certificate.ApprovedByUser;
            logCertificate.SignedByUser = certificate.SignedByUser;
            logCertificate.Number = certificate.Number;
            logCertificate.LastSavedTime = certificate.LastSavedTime;

            return logCertificate;
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateSave, Authenticated = true)]
        public PropertyCertificate ChangeStatus(PropertyCertificate certificate,
            PropertyCertificateStatusCodes newStatus)
        {
            PropertyCertificateRules.ValidateChangeStatus(certificate, newStatus);
            if (certificate.WorkspaceID != CurrentUser.Workspace.ID)
            {
                throw new InvalidWorkspaceException(INVALID_WORKSPACE);
            }

            certificate.Status = GetStatus(newStatus);
            switch (newStatus)
            {
                case PropertyCertificateStatusCodes.Approved:
                    certificate.ApprovedByUser = CurrentUser;
                    break;
                case PropertyCertificateStatusCodes.Signed:
                    certificate.SignedByUser = CurrentUser;
                    break;
            }

            return certificate;
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateView, Authenticated = true)]
        public PropertyCertificate[] GetList()
        {
            IQuery listQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificate as pc where pc.WorkspaceID = :WorkspaceID");
            listQuery.SetInt32("WorkspaceID", CurrentUser.Workspace.ID);
            IList propertyCertificateList = listQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificate>(propertyCertificateList);
        }

        [WebMethod]
        public PropertyCertificateStatus GetStatus(PropertyCertificateStatusCodes status)
        {
            return SessionContext.Session.Load<PropertyCertificateStatus>((int)status);
        }

        [WebMethod]
        public PropertyCertificateStatus[] GetStatusList()
        {
            IQuery statusQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificateStatus");
            IList statusList = statusQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificateStatus>(statusList);
        }

        [WebMethod]
        public PropertyCertificateType[] GetTypeList()
        {
            IQuery typeQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificateType");
            IList typeList = typeQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificateType>(typeList);
        }

        [WebMethod]
        public PropertyCertificateIssuer[] GetIssuerList()
        {
            IQuery typeQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificateIssuer");
            IList typeList = typeQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificateIssuer>(typeList);
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateView, Authenticated = true)]
        public PropertyCertificate[] SearchByNumber(int number)
        {
            IQuery searchQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificate as pc where (pc.Number = :Number) AND (pc.WorkspaceID = :WorkspaceID)");
            searchQuery.SetInt32("Number", number);
            searchQuery.SetInt32("WorkspaceID", CurrentUser.Workspace.ID);

            IList resultsList = searchQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificate>(resultsList);
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateView, Authenticated = true)]
        public PropertyCertificate[] SearchByDateCreated(DateTime fromDate,
            DateTime toDate)
        {
            IQuery searchQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificate as pc " +
                "where ((pc.DateCreated >= :FromDate) AND (pc.DateCreated <= :ToDate) " +
                "  AND (pc.WorkspaceID = :WorkspaceID))");
            searchQuery.SetDateTime("FromDate", AppUtils.TrimTime(fromDate));
            searchQuery.SetDateTime("ToDate", AppUtils.FillMaxTime(toDate));
            searchQuery.SetInt32("WorkspaceID", CurrentUser.Workspace.ID);

            IList resultsList = searchQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificate>(resultsList);
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateView, Authenticated = true)]
        public PropertyCertificate[] SearchByStatus(int statusID)
        {
            IQuery searchQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificate as pc " +
                "where (pc.Status.ID = :ID) AND (pc.WorkspaceID = :WorkspaceID)");
            searchQuery.SetInt32("ID", statusID);
            searchQuery.SetInt32("WorkspaceID", CurrentUser.Workspace.ID);

            IList resultsList = searchQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificate>(resultsList);
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateView, Authenticated = true)]
        public PropertyCertificate[] SearchByApprovedUser(string fullname)
        {
            IQuery searchQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificate as pc " +
                "where (pc.ApprovedByUser.Fullname like :Name) " +
                "  AND (pc.WorkspaceID = :WorkspaceID)");
            searchQuery.SetString("Name", fullname + "%");
            searchQuery.SetInt32("WorkspaceID", CurrentUser.Workspace.ID);

            IList resultsList = searchQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificate>(resultsList);
        }

        [WebMethod]
        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateView, Authenticated = true)]
        public PropertyCertificate[] SearchBySignedUser(string fullname)
        {
            IQuery searchQuery = SessionContext.Session.CreateQuery(
                "from PropertyCertificate as pc " +
                "where (pc.SignedByUser.Fullname like :Name) " +
                "  AND (pc.WorkspaceID = :WorkspaceID)");
            searchQuery.SetString("Name", fullname + "%");
            searchQuery.SetInt32("WorkspaceID", CurrentUser.Workspace.ID);

            IList resultsList = searchQuery.List();

            return AppUtils.ListToFixedArray<PropertyCertificate>(resultsList);
        }

        [WebMethod]
        public LogPropertyCertificate[] SearchLogPropertyCertificate(int userID, string actionType, DateTime dateFrom, DateTime dateTo)
        {
            return SearchLog<LogPropertyCertificate>("LogPropertyCertificate", userID, actionType, dateFrom, dateTo);
        }

        T[] SearchLog<T>(string entity, int userID, string actionType, DateTime dateFrom, DateTime dateTo)
        {
            IList result = SessionContext.Session.Find(
                "SELECT entry FROM " + entity + " AS entry WHERE " +
                    " entry.LogDate >= ? AND entry.LogDate <= ? AND " +
                    " entry.LogActionType = COALESCE(?, entry.LogActionType) AND " +
                    " entry.UserActualize.ID = COALESCE(?, entry.UserActualize.ID) ",
                new object[] {AppUtils.TrimTime(dateFrom),
                    AppUtils.FillMaxTime(dateTo),
                    AppUtils.EmptyToNull(actionType), 
                    AppUtils.ZeroToNull(userID)},
                new NHibernate.Type.IType[] { 
                    NHibernateUtil.DateTime, 
                    NHibernateUtil.DateTime,
                    NHibernateUtil.String,
                    NHibernateUtil.Int32
                });

            return AppUtils.ListToFixedArray<T>(result);
        }

    }
}
